@@ -6,6 +6,6 @@ export PIP_IGNORE_INSTALLED=False
 
 $PYTHON -m pip install keras-tuner -vv --no-dependencies
 $PYTHON -m pip install kt-legacy -vv --no-dependencies
-$PYTHON -m pip install tensorflow-macos==2.8 tensorflow-metal
+$PYTHON -m pip install tensorflow-macos tensorflow-metal
 $PYTHON -m pip install das -vv --no-dependencies
 
