@@ -486,12 +486,12 @@ def predict(x: np.ndarray,
     class_probabilities = predict_probabilities(x, model, params, verbose, prepend_data_padding)
 
     if pad:
-        # trim probs to original len of x
-        class_probabilities = class_probabilities[:x_len_original, :]
         # set all song probs in padded section to zero to avoid out of bounds detections!
         # assumes that the non-song class is at index 0
-        # class_probabilities[-pad_len:, 1:] = 0
-        # class_probabilities[-pad_len:, 0] = 1
+        class_probabilities[-pad_len:, 1:] = 0
+        class_probabilities[-pad_len:, 0] = 1
+        # trim probs to original len of x
+        class_probabilities = class_probabilities[:x_len_original, :]
 
     events, segments = predict_song(class_probabilities=class_probabilities,
                                     params=params,
@@ -505,12 +505,18 @@ def predict(x: np.ndarray,
                                     segment_minlen=segment_minlen,
                                     segment_fillgap=segment_fillgap)
     if not save_memory:
-        segments['probabilities'] = segments['probabilities'].compute()
-        segments['samples'] = segments['samples'].compute()
-        class_probabilities = class_probabilities.compute()
+        segments['probabilities'] = _to_np(segments['probabilities'])
+        segments['samples'] = _to_np(segments['samples'])
+        class_probabilities = _to_np(class_probabilities)
     return events, segments, class_probabilities, params['class_names']
 
 
+def _to_np(array):
+    if isinstance(array, dask.array.core.Array):
+        array = array.compute()
+    return array
+
+
 def cli_predict(path: str,
                 model_save_name: str,
                 *,
