@@ -5,7 +5,6 @@ import flammkuchen as fl
 import numpy as np
 from tensorflow.keras.callbacks import EarlyStopping, ModelCheckpoint, ReduceLROnPlateau, TensorBoard
 from tensorflow import keras
-import sklearn.utils
 import os
 import yaml
 from typing import List, Optional, Tuple, Dict, Any, Union
@@ -70,15 +69,16 @@ def train(*,
           balance: bool = False,
           version_data: bool = True,
           post_opt: bool = False,
-          fill_gaps_min: float = 0.0005,
-          fill_gaps_max: float = 1.0,
-          fill_gaps_steps: int = 20,
-          min_len_min: float = 0.0005,
-          min_len_max: float = 1.0,
-          min_len_steps: int = 20,
+          post_opt_nb_workers: int = -1,
+          post_opt_fill_gaps_min: float = 0.0005,
+          post_opt_fill_gaps_max: float = 1.0,
+          post_opt_fill_gaps_steps: int = 20,
+          post_opt_min_len_min: float = 0.0005,
+          post_opt_min_len_max: float = 1.0,
+          post_opt_min_len_steps: int = 20,
           resnet_compute: bool = False,
           resnet_train: bool = False,
-          _qt_progress: bool = False) -> Tuple[keras.Model, Dict[str, Any]]:
+          _qt_progress: bool = False) -> Tuple[keras.Model, Dict[str, Any], keras.callbacks.History]:
     """Train a DAS network.
 
     Args:
@@ -192,18 +192,20 @@ def train(*,
 
         post_opt (bool): Optimize post processing (delete short detections, fill brief gaps).
                         Defaults to False.
-        fill_gaps_min (float): Defaults to 0.0005 seconds.
-        fill_gaps_max (float): Defaults to 1 second.
-        fill_gaps_steps (int): Defaults to 20.
-        min_len_min (float): Defaults to 0.0005 seconds.
-        min_len_max (float): Defaults to 1 second.
-        min_len_steps (int): Defaults to 20.
+        post_opt_nb_workers (int): Number of parallel processes during post_opt. Defaults to -1 (same number as cores).
+        post_opt_fill_gaps_min (float): Defaults to 0.0005 seconds.
+        post_opt_fill_gaps_max (float): Defaults to 1 second.
+        post_opt_fill_gaps_steps (int): Defaults to 20.
+        post_opt_min_len_min (float): Defaults to 0.0005 seconds.
+        post_opt_min_len_max (float): Defaults to 1 second.
+        post_opt_min_len_steps (int): Defaults to 20.
 
         resnet_compute (bool): Defaults to False.
         resnet_train (bool): Defaults to False.
         Returns
             model (keras.Model)
             params (Dict[str, Any])
+            history (keras.callbacks.History)
         """
     # _qt_progress: tuple of (multiprocessing.Queue, threading.Event)
     #        The queue is used to transmit progress updates to the GUI,
@@ -367,6 +369,7 @@ def train(*,
     if save_name is None:
         save_name = time.strftime('%Y%m%d_%H%M%S')
     save_name = '{0}/{1}{2}'.format(save_dir, save_prefix, save_name)
+    params['save_name'] = save_name
     logger.info(f'Will save to {save_name}.')
 
     # SET UP CALLBACKS
@@ -410,13 +413,14 @@ def train(*,
     if post_opt:
         logger.info('OPTIMIZING POSTPROCESSING:')
 
-        gap_durs = np.geomspace(fill_gaps_min, fill_gaps_max, fill_gaps_steps)
-        min_lens = np.geomspace(min_len_min, min_len_max, min_len_steps)
+        gap_durs = np.geomspace(float(post_opt_fill_gaps_min), float(post_opt_fill_gaps_max), int(post_opt_fill_gaps_steps))
+        min_lens = np.geomspace(float(post_opt_min_len_min), float(post_opt_min_len_max), int(post_opt_min_len_steps))
 
         best_gap_dur, best_min_len, scores = postprocessing.optimize(dataset_path=data_dir,
                                                                      model_save_name=save_name,
                                                                      gap_durs=gap_durs,
-                                                                     min_lens=min_lens)
+                                                                     min_lens=min_lens,
+                                                                     nb_workers=post_opt_nb_workers)
 
         logger.info(f"  Score on training data changed from {scores['train_pre']:1.4} to {scores['train']:1.4}.")
         if scores['val_pre'] is not None:
@@ -455,6 +459,10 @@ def train(*,
         conf_mat, report = evaluate.evaluate_segments(labels_test, labels_pred, params['class_names'], report_as_dict=True)
         logger.info(conf_mat)
         logger.info(report)
+        params['conf_mat'] = conf_mat.tolist()
+        params['report'] = report
+        logger.info(f'   Updating params file "{save_name}_params.yaml" with the test results.')
+        utils.save_params(params, save_name)
 
         if wandb_api_token and wandb_project:  # could also get those from env vars!
             wandb.log_test_results(report)
@@ -475,4 +483,4 @@ def train(*,
         fl.save(save_filename, ddd)
 
     logger.info('DONE.')
-    return model, params
+    return model, params, fit_hist
